Unit Maths;

Interface

uses Windows,SysUtils;

const
     MaxVecteur = 8192;
     degreDer = 2;
     ordreDer = degreDer+1;
     nbrePointDer = 5;

type
     IndiceVecteur = 0..MaxVecteur;
     TVecteur = array[IndiceVecteur] of double;
     TVecteurInt = array[IndiceVecteur] of integer;
     vectorPuiss = array[0..2*degreDer] of double;
     matrice = array[1..ordreDer,1..ordreDer+1] of double;
     matriceLigne = array[1..ordreDer] of double;
     EsystemeLineaireError = class(Exception);

const
     SansSignif : double = 1e-26;
     Loge = 0.434294481903251828;
     Ln10 = 2.302585092994045680;

Function Dix(x : integer) : double;
Function Log(x : double) : double;
Function MyInt(x : double) : double;
Function Plancher(x : double) : LongInt;
Function Plafond(x : double) : LongInt;
Function Fract(x : double) : double;

procedure DeriveeGauss(const X,Yr : Tvecteur;N : integer;var Yder : Tvecteur);

Implementation

Function PuissEnt(X:double;N:Integer): double;

  Function PuissDeux(N:integer):double; { N>=0 }
  var Y : double;
  begin
     if N=0
        then PuissDeux := 1
        else begin
           Y := PuissDeux(N div 2);
	   Y := Sqr(Y);
 	   if (N mod 2)=0
	      then PuissDeux := Y
	      else PuissDeux := X*Y;
       end;
  end;{ puissDeux }

begin {PuissEnt}
    If N<0
       then PuissEnt := 1/PuissDeux(-N)
       else PuissEnt := PuissDeux(N)
end;{PuissEnt}

function Dix(x : integer) : double;
Begin
    Dix := PuissEnt(10,X)
end;

Function Log(X:double):double;
Begin
   Log := Ln(X)*loge
End;

function Plancher(x : double) : LongInt;
var zut : LongInt;
begin
     Zut := trunc(x);
     if (x>=0) or (zut=x)
        then Plancher := zut
        else Plancher := zut-1
end;

function MyInt(x : double) : double;
var zut : double;
begin
    Zut := int(x);
    if (x>=0) or (zut=x)
       then Myint := zut
       else Myint := zut-1
end;

function Plafond(x : double) : LongInt;
var zut : LongInt;
begin
     Zut := trunc(x);
     if (x<0) or (Zut=x)
       then Plafond := zut
       else Plafond := zut+1
end;

function Fract(x : double) : double;
var zut : double;
begin
     Zut := frac(x);
     if (x>=0) or (zut=0)
        then Fract := zut
        else Fract := zut+1
end;

Procedure Resolution(A : matrice;ordre : byte;var Result : matriceLigne);
 { Cette procdure rsoud un systme AX = B }
 { crit sous la forme AX = A(n+1) }
 { par la mthode d'limination de Gauss (pivot partiel) }

  Procedure Elimination;

    FUNCTION Pivot(i :integer):integer;
    Var n,temp : integer;
    BEGIN
      temp := i;
      for n := i+1 to ordre do
          if abs(A[n,i]) > abs(A[i,i]) then temp:= n;
      pivot:=temp;
      if abs(A[temp,i]) <= 1e-100 then
         raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
    END;

    PROCEDURE Permute(i,j:integer);
    Var temp:double;
        k:integer;
    BEGIN
       if i<>j then For k:=i to ordre+1 do BEGIN
          temp:=A[i,k];
          A[i,k]:=A[j,k];
          A[j,k]:=temp;
       END;
    END;

  var I,J,K : integer;
      M : double;
  begin
    for k := 1 to ordre do begin
          Permute(k,pivot(k));
          for i:=k+1 to ordre do begin
              M := A[i,k]/A[k,k];
              for j:=k+1 to ordre+1 do A[i,j]:=A[i,j]-M*A[k,j];
              A[i,k]:=0.0;
          end;
       end;
    end; {Elimination}

    Procedure Substitution;
    var I,J : integer;
        S : double;
    begin
         try
         for I:=ordre downto 1 do begin
             S:=A[I,ordre+1];
             for J:=I+1 to ordre do S:=S-A[I,J]*Result[J];
             Result[I]:=S/A[I,I];
         end;
         except
            on EzeroDivide do raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
         end;
    end;

begin {Resolution}
  Elimination;
  Substitution;
end; {rsolution}

procedure DeriveeGauss(const X,Yr : Tvecteur;N : integer;var Yder : Tvecteur);
var puissX : vectorPuiss;
    A : matrice;
    Coeff : matriceLigne;
    ii,puissMax,ordre : byte;

procedure initXY;
var j : integer;
    loc : double;
    i : byte;
begin
    for i := 1 to puissMax do puissX[i] := 0;
    puissX[0] := NbrePointDer;
    for i := 1 to ordre do A[i,ordre+1] := 0;
    for j := 0 to pred(NbrePointDer) do begin
        loc := x[j];
        for i := 1 to puissMax do begin
            puissX[i] := puissX[i]+loc;
            loc := loc*x[j];
        end;
        loc := yr[j];
        for i := 1 to ordre do begin
            A[i,ordre+1] := A[i,ordre+1]+loc;
            loc := loc*x[j];
        end;
    end;
end;

Procedure CalculXY(i : integer);
var ajoute,retire : double;
    k : byte;
begin
        ajoute := X[i+ii];
        retire := X[i-ii-1];
        for k := 1 to puissMax do begin
            puissX[k] := puissX[k]+ajoute-retire;
            retire := retire*X[i-ii-1];
            ajoute := ajoute*X[i+ii];
        end;
        ajoute := Yr[i+ii];
        retire := Yr[i-ii-1];
        for k := 1 to ordre do begin
            A[k,ordre+1] := A[k,ordre+1]+ajoute-retire;
            retire := retire*X[i-ii-1];
            ajoute := ajoute*X[i+ii];
        end;
end;

procedure AffecteCoeff;
var i,j : byte;
begin
     for i := 1 to ordre do
         for j := 1 to ordre do
             A[i,j] := PuissX[i+j-2];
     try
     Resolution(A,ordre,Coeff);
     except
         coeff[2] := sansSignif;
         coeff[3] := 0;
     end;
end;

Procedure AffecteDerivee(i : integer);
begin
   case degreDer of
        1 : yder[i] := coeff[2];
        2 : yder[i] := coeff[2]+2*coeff[3]*x[i];
   end;
end;

var i : integer;
begin { DeriveeGauss }
    for i := 0 to pred(N) do yder[i] := 0;
    if N<NbrePointDer then exit;
    ordre := degreDer+1;
    puissMax := 2*degreDer;
    ii := NbrePointDer div 2;
    initXY;
    affecteCoeff;
    for i := 0 to ii do affecteDerivee(i);
    for i := ii+1 to N-ii-1 do begin
        calculXY(i);
        affecteCoeff;
        affecteDerivee(i);
    end;
    for i := N-ii to pred(N) do affecteDerivee(i);
end; { DeriveeGauss }


end.

