unit TableGraphe;

interface

uses windows, sysUtils, graphics, classes,
     math, maths, tableUtil, uniteKer, tableDeF, printers, dialogs, matrices;

const
        scaleCurseur = 9;
        scaleTrace = 5;
        voieT = 0;
        voieX = 1;
        voieY = 2;
        maxTrace = 63;
        NbreVecteurVitesse = 20;
        MondeVitesse = 1;
        MondeAcc = 2;
        EchelleVecteur : integer = 10;
type
        indiceMondeVecteur = mondeVitesse..MondeAcc;
        TetatStylet = (esPoint1,esPoint2,esPoint3,esAttente,esAcquisition);

const
        couleurVecteur : array[indiceMondeVecteur] of Tcolor = (clGreen,clRed);
        nomStylet : array[TetatStylet] of string[10] =
          ('premier','deuxime','troisime','','');

type
        indiceVoie = voieT..VoieY;
        Tcourbe = class(Tvoie)
                 a,b : double; { entier = rel*a+b }
	             constructor create(Anom : strUnite;Aunite: strUnite;
                 Asignif : shortString);
                 Function XWindow(r : double) : integer;
                 Function XMonde(xi : integer) : double;
       	end;

        TmondeVecteur = class
         maxi : double;
         vecteurX,vecteurY : Tvecteur;
         ax,ay : double; { entier = rel*a }
         echelle : double;
         trace,defini : boolean;
         constructor create;
         procedure Trouve(NbreM : integer);
         procedure GetEntier(x,y : double;var xi,yi : integer);
      end;



	Tgraphe = class
           Canvas : Tcanvas;
       	 traceLigne,traceMotif : boolean;
       	 voies : array[indiceVoie] of Tcourbe;
       	 LimiteCourbe : Trect;
           Nbre : integer;
           reticule : Tpoint;
           pixel : boolean;
           avecTrace : boolean;
           dimPoint : integer;
           avecStylet : boolean;
           arretProxi : boolean;
           vitesse : Tvitesse;
           ImprimanteEnCours : boolean;
           indexImprimante : integer;
           PixelsMetreX,PixelsMetreY : integer;
           mondeVecteur : array[indiceMondeVecteur] of TmondeVecteur;
           Trace : array[0..maxTrace] of Tpoint;
           Yt,Xt,Yd,Xd : array[1..3] of double;
           deltatStylet : double;
           x0,y0,axx,axy,ayx,ayy : double; // coeff de la matrice de passage de la table vers les donnes
     	   Constructor Create;
       	   Procedure Draw;
           Procedure VersEcran(Acanvas : Tcanvas;largeur,hauteur : integer);
           Procedure TraceReticule;
           Procedure SetTrace;
           procedure VersImprimante;
           Procedure Segment(x1,y1,x2,y2 : integer);
           Procedure Motif(x,y : integer);
           Procedure ConversionStylet(xt,yt : double;var xd,yd : double);
           Procedure CalculeConversionStylet;
           Procedure AjoutePointCourant;
           destructor Destroy; override;
	 end;

IMPLEMENTATION

Function Tcourbe.XMonde(xi : integer) : double;
begin
	result := (xi-B)/A
end;

Function Tcourbe.XWindow(r : double) : integer;
begin
      if r=sansSignif
         then result := -1
         else try
            result := round(A*r+B)
            except
               result := -1
            end
end;

procedure Tgraphe.TraceReticule;

Procedure TraceLoc;
begin
      if (reticule.x>0) and (reticule.y>0) then with canvas do begin
             moveTo(reticule.x-scaleCurseur,reticule.y);
             lineTo(reticule.x+scaleCurseur,reticule.y);
             moveTo(reticule.x,reticule.y-scaleCurseur);
             lineTo(reticule.x,reticule.y+scaleCurseur);
      end;
end;

begin
      if avecTrace then setTrace else begin
      canvas.pen.color := clBlue;
      canvas.pen.mode := pmNotXor;
      canvas.pen.width := 3;
      traceLoc;
      reticule.x := voies[voieX].XWindow(voies[voieX].valeurCourante);
      reticule.y := voies[voieY].XWindow(voies[voieY].valeurCourante);
      traceLoc
end end;{ TraceReticule }

procedure Tgraphe.SetTrace;
const Increment = 4;
var i,couleur : integer;
begin with canvas do begin
      pen.width := 2;
      pen.Mode := pmCopy;
      brush.style := bsClear;
      for i := 0 to pred(maxTrace) do trace[i] := trace[i+1];
      trace[maxTrace].x := voies[voieX].XWindow(voies[voieX].valeurCourante);
      trace[maxTrace].y := voies[voieY].XWindow(voies[voieY].valeurCourante);
      for i := 0 to maxTrace do if (trace[i].x>scaleTrace) and (trace[i].y>scaleTrace) then begin
             couleur := 255-i*increment;
             pen.color := RGB(couleur,couleur,255);
             ellipse(trace[i].x-scaleTrace,trace[i].y-scaleTrace,
                     trace[i].x+scaleTrace,trace[i].y+scaleTrace);
      end;
end end;{ setTrace }

Procedure Tgraphe.Segment(x1,y1,x2,y2 : integer);
begin
     Canvas.moveTo(x1,y1);
     Canvas.lineTo(x2,y2);
end;

Procedure Tgraphe.Motif(x,y : integer);
begin
     if (x>0) and (y>0) then if pixel
        then canvas.pixels[x,y] := clBlue
        else begin
            Canvas.moveTo(x-dimPoint,y);
            Canvas.lineTo(x+dimPoint,y);
            Canvas.moveTo(x,y-dimPoint);
            Canvas.lineTo(x,y+dimPoint);
//           Canvas.ellipse(x-dimPoint,y-dimPoint,x+dimPoint,y+dimPoint);
        end;
end;

procedure Tgraphe.Draw;
var margeX,margeY : integer;

   Procedure ChercheMondeVecteur;
   var dimension,m : integer;
   begin
       mondeVecteur[mondeAcc].defini := false;
       mondeVecteur[mondeVitesse].defini := false;
       if Nbre<NbreVecteurVitesse then exit;
       with limiteCourbe do Dimension := (bottom-top)+(right-left);
       if mondeVecteur[mondeVitesse].trace or mondeVecteur[mondeAcc].trace then begin
          try
          DeriveeGauss(voies[voieT].vecteur,
                    voies[voieX].vecteur,Nbre,mondeVecteur[mondeVitesse].vecteurX);
          DeriveeGauss(voies[voieT].vecteur,
                    voies[voieY].vecteur,Nbre,mondeVecteur[mondeVitesse].vecteurY);
          mondeVecteur[mondeVitesse].trouve(Nbre);
          except
              mondeVecteur[mondeVitesse].defini := false;
          end
       end;
       if mondeVecteur[mondeAcc].trace and mondeVecteur[mondeVitesse].defini then begin
          try
          DeriveeGauss(voies[voieT].vecteur,
                    mondeVecteur[mondeVitesse].vecteurX,Nbre,mondeVecteur[mondeAcc].vecteurX);
          DeriveeGauss(voies[voieT].vecteur,
                        mondeVecteur[mondeVitesse].vecteurY,Nbre,mondeVecteur[mondeAcc].vecteurY);
          mondeVecteur[mondeAcc].trouve(Nbre);
          except
             mondeVecteur[mondeAcc].defini := false;
          end;
       end;
       for m := low(indiceMondeVecteur) to high(indiceMondeVecteur) do
           with mondeVecteur[m] do
              if trace and defini then if -voies[voieY].a>voies[voieX].a
                 then begin
                    ax := dimension*EchelleVecteur/maxi/100;
                    ay := ax*voies[voieY].a/voies[voieX].a;
                  end
                  else begin
                    ay := -dimension*EchelleVecteur/maxi/100;
                    ax := ay*voies[voieX].a/voies[voieY].a;
                  end;
   end;{ ChercheMondeVecteur }

procedure ChercheMonde;
begin
     with voies[voieX] do begin
           A := (limiteCourbe.right-limiteCourbe.left)/Maxi;
           B := limiteCourbe.left;
     end;
     with voies[voieY] do begin
           A := (limiteCourbe.Top-limiteCourbe.Bottom)/Maxi;
           B := limiteCourbe.Bottom;
     end;
// passage en mode orthonorm
     if abs(voies[voieY].A)>abs(voies[voieX].A)
         then begin
              voies[voieY].A := -voies[voieX].A
         end
         else begin
             voies[voieX].A := -voies[voieY].A
         end;
     canvas.pen.width := 1;
     canvas.pen.mode := pmCopy;
     canvas.pen.color := clBlack;
// trac partie utile
     limiteCourbe.right := voies[voieX].XWindow(voies[voieX].maxi);
     limiteCourbe.top := voies[voieY].XWindow(voies[voieY].maxi);
     canvas.Rectangle(limiteCourbe);
end; { ChercheMonde }

procedure ChercheMondeImprimante;
var TraceMax,TraceMin,trace0 : double;
    i : integer;
begin
     with voies[voieX] do begin
           A := PixelsMetreX;
           B := limiteCourbe.left;
           if XWindow(voies[voieX].maxi)>limiteCourbe.Right then begin
                // recherche partie utile
              TraceMax := vecteur[0];
              TraceMin := vecteur[0];
              for i := 1 to pred(Nbre) do
                  if vecteur[i]>TraceMax
                      then TraceMax := vecteur[i]
                      else if vecteur[i]<TraceMin then TraceMin := vecteur[i];
              trace0 := (TraceMax+TraceMin)/2;//  mettre au milieu de la feuille
              B := ((limiteCourbe.left+limiteCourbe.right) div 2)-A*trace0;

           end;
     end;
     with voies[voieY] do begin
           A := -PixelsMetreY;
           B := limiteCourbe.Bottom;
           if XWindow(voies[voieX].mini)>limiteCourbe.bottom then begin
                // recherche partie utile
              TraceMax := vecteur[0];
              TraceMin := vecteur[0];
              for i := 1 to pred(Nbre) do
                  if vecteur[i]>TraceMax
                      then TraceMax := vecteur[i]
                      else if vecteur[i]<TraceMin then TraceMin := vecteur[i];
              trace0 := (TraceMax+TraceMin)/2;//  mettre au milieu de la feuille
              B := ((limiteCourbe.top+limiteCourbe.bottom) div 2)-A*trace0;
           end;
     end;
     canvas.pen.width := 1;
     canvas.pen.mode := pmCopy;
     canvas.pen.color := clBlack;
end; { ChercheMondeImprimante }

Procedure TracePoints;
var i : indiceVecteur;
    xi,yi : integer;
begin
   for i := 0 to pred(Nbre) do begin
       xi := voies[voieX].XWindow(voies[voieX].vecteur[i]);
       yi := voies[voieY].XWindow(voies[voieY].vecteur[i]);
       motif(xi,yi);
   end;{i}
end;{tracePoints}

Procedure TraceRepere;
var i : integer;
    xi,yi : integer;
begin
   for i := 1 to 3 do if (Xt[i]*Yt[i]>0) then begin
       xi := voies[voieX].XWindow(Xt[i]);
       yi := voies[voieY].XWindow(Yt[i]);
       segment(xi-4,yi-4,xi+4,yi+4);
       segment(xi-4,yi+4,xi+4,yi-4);       
       canvas.textOut(xi-margeX*3,yi+6,'Point '+intToStr(i));
   end;{i}
end;{tracePoints}

Procedure traceVecteur(m : byte);
var xi,yi : Integer;
    i : integer;
    h : integer; // dimension en pixel de la flche tq 1% de la largeur

Procedure TraceFleche(dX,dY : double);
var x,y,d : integer;
    deltaX,deltaY : integer;
    Points : array[0..2] of Tpoint;
begin
      MondeVecteur[m].getEntier(dX,dY,deltaX,deltaY);
      canvas.MoveTo(xi,yi);
      yi := yi+deltaY;
      xi := xi+deltaX;
      canvas.LineTo(xi,yi);
      d := round(sqrt(sqr(deltaX)+sqr(deltaY)));
      deltaX := round(h*deltaX/d);
      deltaY := round(h*deltaY/d);
  { x,y = centre de la base de la flche }
      y := yi-deltaY;
      x := xi-deltaX;
      deltaX := round(deltaX/3);
      deltaY := round(deltaY/3);
      Points[0] := Point(xi,yi);
      Points[1] := Point(x-deltaY,y+deltaX);
      Points[2] := Point(x+deltaY,y-deltaX);
      canvas.Polygon(Points);
end;

var pas : integer;
    dXr,dYr : double;
begin {traceVecteur}
     try
     h := (limiteCourbe.Right-LimiteCourbe.Left) div 100;
     if h<8 then h := 8;
     canvas.Brush.color := couleurVecteur[m];
     canvas.pen.color := couleurVecteur[m];
     pas := succ(Nbre div NbreVecteurVitesse);
     for i := 2 to Nbre-3 do if (i mod pas)=0 then begin
        xi := voies[voieX].XWindow(voies[voieX].vecteur[i]);
        yi := voies[voieY].XWindow(voies[voieY].vecteur[i]);
        dXr := mondeVecteur[m].vecteurX[i];
        dYr := mondeVecteur[m].vecteurY[i];
        traceFleche(dXr,dYr);
     end;{for i}
     except
     end;
end;{traceVecteur}

Procedure traceLignes;
var i : indiceVecteur;
    xi,yi : integer;
begin
     if Nbre=0 then exit;
     xi := voies[voieX].XWindow(voies[voieX].vecteur[0]);
     yi := voies[voieY].XWindow(voies[voieY].vecteur[0]);
     canvas.MoveTo(xi,yi);
     for i := 1 to pred(Nbre) do begin
        xi := voies[voieX].XWindow(voies[voieX].vecteur[i]);
        yi := voies[voieY].XWindow(voies[voieY].vecteur[i]);
        canvas.LineTo(xi,yi);
     end;{for i}
end;{traceLignes}

var i : integer;
    deltat,dt : double;
begin
     if imprimanteEnCours
        then chercheMondeImprimante
        else chercheMonde;
     chercheMondeVecteur;   
     pixel := dimPoint = 0;
     margeX := canvas.TextWidth('A');
     margeY := canvas.TextHeight('A') div 2;
     Canvas.Font.Color := clBlack;
     Canvas.Font.Size := 10;
     reticule.x := -1;
     reticule.y := -1;
     for i := 0 to maxTrace do begin
         trace[i].x := -1;
         trace[i].y := -1;
     end;
     if avecStylet then TraceRepere;
     if Nbre=0 then exit;
     canvas.pen.width := 1;
     canvas.pen.mode := pmCopy;
     canvas.pen.color := clBlue;
     if Nbre>8 then begin
        deltat := 1;
        for i := 1 to pred(Nbre) do begin
            dt := (voies[voieT].vecteur[i]-voies[voieT].vecteur[pred(i)]);
            if dt<deltat then deltat := dt;
        end;
        canvas.textOut(limiteCourbe.Left+margeX,limiteCourbe.top+margeY,
                  'dt(mini)='+IntToStr(round(1000*deltat))+' ms');
     end;             
     if traceLigne then TraceLignes;
     if traceMotif then TracePoints;
     for i := low(indiceMondeVecteur) to high(indiceMondeVecteur) do
         if mondeVecteur[i].trace and mondeVecteur[i].defini then traceVecteur(i);
end; {Tgraphe.draw}

procedure TGraphe.VersImprimante;
var sauveDimPoint : integer;
    PixelsPerInchX,PixelsPerInchY : integer;
begin
     try
     if (indexImprimante>=0) and
        (indexImprimante<printer.printers.count) then
            printer.printerIndex := indexImprimante;
     printer.title := 'Dig-Dedoc';
     Printer.Canvas.Font.Name := 'Arial Regressi';
     Printer.Canvas.Font.Size := 14;
     Printer.Canvas.Font.Color := clBlack;
     printer.beginDoc;
     sauveDimPoint := dimPoint;
     PixelsPerInchX := GetDeviceCaps(Printer.Canvas.Handle,LOGPIXELSX);
     PixelsPerInchY := GetDeviceCaps(Printer.Canvas.Handle,LOGPIXELSY);
     PixelsMetreX := round(PixelsPerInchX/0.0254); // 1 inch = 1 pouce = 2.54 cm
     PixelsMetreY := round(PixelsPerInchY/0.0254);
     dimPoint := dimPoint*PixelsMetreX div 4000;
     // 4000=1/2.5e-4 ; 0,25 mm = dim pixel cran
     limiteCourbe := Printer.Canvas.ClipRect;
     ImprimanteEnCours := true;
     canvas := printer.Canvas;
     Draw;
     printer.endDoc;
     ImprimanteEnCours := false;
     dimPoint := sauveDimPoint;
     except
        showMessage('Vrifier l''imprimante slectionne dans les options');
     end;
end;

procedure Tgraphe.AjoutePointCourant;
var xi,yi : integer;
var v : indiceVoie;
begin
     for v := voieT to voieY do
         voies[v].vecteur[Nbre] := voies[v].ValeurCourante;
     canvas.pen.width := 1;
     canvas.pen.mode := pmCopy;
     canvas.pen.color := clBlue;
     xi := voies[voieX].XWindow(voies[voieX].vecteur[Nbre]);
     yi := voies[voieY].XWindow(voies[voieY].vecteur[Nbre]);
     if traceMotif then motif(xi,yi);
     if traceLigne then if Nbre=0
        then canvas.moveTo(xi,yi)
        else canvas.lineTo(xi,yi);
     inc(Nbre);
end; {Tgraphe.ajoutePoint}

destructor Tgraphe.destroy;
var i : indiceVoie;
begin
     for i := voieT to VoieY do voies[i].free;
     for i := low(indiceMondeVecteur) to high(indiceMondeVecteur) do
         mondeVecteur[i].free;
     inherited destroy;
end;

constructor Tgraphe.Create;
var m : integer;
begin
  inherited create;
  traceLigne := true;
  indexImprimante := -1;
  traceMotif := false;
  voies[voieT] := Tcourbe.create('t','s','temps');
  voies[voieT].Maxi := 30;
  voies[voieX] := Tcourbe.create('x','m','abscisse');
  voies[voieX].Maxi := 0.25;
  voies[voieY] := Tcourbe.create('y','m','ordonne');
  voies[voieY].Maxi := 0.25;
  for m := low(indiceMondeVecteur) to high(indiceMondeVecteur) do
      mondeVecteur[m] := TmondeVecteur.create;
  Xt[1] := 0;  Yt[1] := 0;  Xd[1] := 0;  Yd[1] := 0;
  Xt[2] := 1;  Yt[2] := 0;  Xd[2] := 1;  Yd[2] := 0;
  Xt[3] := 0;  Yt[3] := 1;  Xd[3] := 0;  Yd[3] := 1;
  deltatStylet := 1;
end;

constructor Tcourbe.create(Anom : strUnite;Aunite: strUnite;
    Asignif : shortstring);
begin
     inherited create;
     nom := Anom;
     NomUnite := Aunite;
     signif := Asignif;
end;

procedure TGraphe.VersEcran(Acanvas : Tcanvas;largeur,hauteur : integer);
begin
     canvas := Acanvas;
     limiteCourbe := rect(0,0,largeur,hauteur);
     ImprimanteEnCours := false;
     Draw;
end;

Procedure TGraphe.ConversionStylet(xt,yt : double;var xd,yd : double);
begin
    xd := x0+axx*xt+axy*yt;
    yd := y0+ayx*xt+ayy*yt;
end;

Procedure TGraphe.CalculeConversionStylet;
var X,Y : Tvector; // Y=A*X
    A,A_inv : Tmatrix;
    i,posX,posY : integer;
    det : float;
begin
    DimVector(X, 5);
    DimVector(Y, 5);
    DimMatrix(A, 5, 5);
    DimMatrix(A_inv, 5, 5); // initialise  zro
// pour les trois points
//     
    A[0,0] := 1;
    A[1,1] := 1;
    A[2,0] := 1;
    A[4,0] := 1;
    A[3,1] := 1;
    A[5,1] := 1;
    for i := 1 to 3 do begin
        posX := 2*i-2;
        posY := posX+1;
        Y[posX] := Xd[i];
        Y[posY] := Yd[i];
        A[posX,2] := Xt[i];
        A[posX,3] := Yt[i];
        A[posY,4] := Xt[i];
        A[posY,5] := Yt[i];
    end;
    if GaussJordan(A,Y,0,5,A_inv,X,Det)=MAT_OK then begin
       x0 := X[0];y0 := X[1];
       axx := X[2];axy := X[3];
       ayx := X[4];ayy := X[5];
    end;
    X := nil;Y := nil;
    A := nil;A_inv := nil;
end;

constructor TmondeVecteur.Create;
begin
    inherited create;
    trace := false;
    defini := false;
end;

const Nbre125 : array[1..3] of integer = (1,2,5);

Procedure TmondeVecteur.Trouve(NbreM : integer);

procedure SetDelta;
const
    indexMax = 3;
    coeff : array[0..indexMax] of double = (10,5,2,1);
var  largeur : double;
     Exponent,index : integer;
begin
    Exponent := floor(Log(Maxi));
    largeur := maxi*dix(-exponent);
    index := 0;
    while (largeur<coeff[index]) and
          (index<indexMax) do
                 inc(index);
    Echelle := coeff[index]*dix(exponent);
end; { setDelta }

var j : integer;
begin
    maxi := 0;
    for j := 0 to pred(NbreM) do begin
        if abs(vecteurX[j])>maxi
           then maxi := abs(vecteurX[j]);
        if abs(vecteurY[j])>maxi
           then maxi := abs(vecteurY[j]);
    end;
    setDelta;
    defini := maxi>0;
end;

procedure TmondeVecteur.getEntier(x,y : double;var xi,yi : Integer);
begin
    xi := round(Ax*x);
    yi := round(Ay*y);
end;

end.

