unit TableUtil;

interface

uses
   SysUtils, Windows, Messages, Classes, Dialogs,
   Graphics, Forms, Consts, Controls,
   Maths, StdCtrls, OOmisc, AwUser, AwWin32, shlObj;

const
   Precision      = 7; // donc
   LongUnite      = 32;
   SymbReg: char  = '';
   SymbReg2: char = '&';
   RegistreTable  = 'Software\TableRegressi';

   alpha     = #128;
   beta      = #129;
   chi       = #130;
   delta     = #131;
   deltaMaj  = #150;
   epsilon   = #132;
   phi       = #133;
   phiMaj    = #151;
   gamma     = #134;
   gammaMaj  = #152;
   eta       = #135;
   plusExp   = #184;
   moinsExp  = #170;
   lambda    = #136;
   lambdaMaj = #153;
   nu        = #138;
   piMin     = #139;
   piMaj     = #154;
   theta     = #140;
   thetaMaj  = #155;
   rho       = #141;
   sigma     = #142;
   sigmaMaj  = #156;
   tau       = #143;
   omega     = #144;
   omegaMaj  = #157;
   xi        = #147;
   xiMaj     = #158;
   psi       = #148;
   psiMaj    = #159;
   zeta      = #149;
   mu        = #181;
   vide      = #248;
   videMaj   = #216;

   ChiffreExp: array['0'..'9'] of char =
      (#186, #185, #178, #179, #165, #169, #172, #173, #174, #188);

type
   ChaineAzt = array[0..255] of char;
   TcharSet  = set of char;
   strUnite  = string

      [LongUnite];
   CoefUnite = (zero, yocto, zepto, atto, femto, pico, nano, micro, milli,
      nulle, kilo, mega, giga, tera, peta, exa, zetta, yotta, infini);

const
   Digits: array[0..$F] of char = '0123456789ABCDEF';
   crSupprArr = char(vk_back);
   crSuppr    = char(vk_clear);
   crHaut     = char(vk_up);
   crBas      = char(vk_down);
   crGauche   = char(vk_left);
   crDroite   = char(vk_right);
   crHome     = char(vk_home);
   crEnd      = char(vk_end);
   crCR       = #13;
   crLF       = #10;
   crESC      = #27;
   crTab      = char(vk_tab);
   Separateur: TCharSet = [' ', crTab, ','];
   Majuscule: TcharSet = ['A'..'Z'];
   Minuscule: TcharSet = ['a'..'z'];
   Grecque: TcharSet =
      [alpha, beta, gamma, gammaMaj, delta, deltaMaj, chi,
      epsilon, phi, phiMaj, eta, psi, psiMaj, lambda, lambdaMaj, mu, nu,
      piMin, piMaj, theta, thetaMaj, rho, sigma, sigmaMaj, tau, omega, omegaMaj,
      xi, xiMaj, zeta];
   ToutCarac: TcharSet = [#032..#151, #224..#250];
   Chiffre: TcharSet = ['0'..'9'];
   ChiffrePuiss: TcharSet = [#186, #185, #178, #179, #165, #169, #172, #173, #174, #188];
   ChiffreEtc: TcharSet = ['0'..'9', '+', '-', 'e', 'E', '.'];
   ChiffreWin: TcharSet = ['0'..'9', '+', '-', 'e', 'E', '.', ','];
   ToucheEdition: TcharSet = [crSupprArr, crGauche, crDroite, crHome, crEnd, crSuppr];

   caracPrefixe: TcharSet =
      ['y', 'z', 'a', 'f', 'p', 'n', '', 'm', 'k', 'M', 'G', 'T', 'P', 'E'];
   caracUnite: TcharSet   =
      ['A'..'Z', 'a'..'z', OmegaMaj, '1'..'3', '-', '.', '/', ''];

   prefixeUnite: array[coefunite] of char =
      ('0', 'y', 'z', 'a', 'f', 'p', 'n', '', 'm',
      ' ', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', '');
   valUnite: array[coefunite] of double =
      (1E-27, 1E-24, 1E-21, 1E-18, 1E-15, 1E-12, 1E-9, 1E-6, 1E-3,
      1, 1E+3, 1E+6, 1E+9, 1E+12, 1E+15, 1E+18, 1E+21, 1E+24, 1E+27);
   puissanceUnite: array[coefunite] of shortint =
      (-27, -24, -21, -18, -15, -12, -9, -6, -3,
      0, +3, +6, +9, +12, +15, +18, +21, +24, +27);

function OKReg(texte: shortString): boolean;
procedure afficheErreur(texte: shortString; code: integer);
function Pad(S: shortString; Len: integer): shortString;
function ConvertitChaine(nombre: strUnite; var valeur: double): boolean;
function FloatToStrPoint(valeur: double): strUnite;
function FormatTable(Nombre: double): strUnite;
function StrToFloatWin(nombre: strUnite): double;
function CaracToChiffre(carac: char): integer;
function PuissToStr(puiss: integer): strUnite;
function IsPortAvailable(ComNum: cardinal): boolean;

var
   winDir, MesDocsDir: string;

implementation

procedure UpCaseStr(var s: shortstring);
var
   i: integer;
begin
   for i := 1 to length(s) do
      s[i] := UpCase(S[i]);
end;

function OKReg(texte: shortString): boolean;
begin
   OKreg := MessageDlg(Texte, mtConfirmation, [mbYes, mbNo], 0) = idYes;
end;

function StrToFloatWin(nombre: strUnite): double;
var
   posSeparator: integer;
begin
   if decimalSeparator <> '.' then begin
      posSeparator := pos('.', nombre);
      if posSeparator > 0 then
         Nombre[posSeparator] := decimalSeparator;
   end
   else if decimalSeparator <> ',' then begin
      posSeparator := pos(',', nombre);
      if posSeparator > 0 then
         Nombre[posSeparator] := decimalSeparator;
   end;
   StrToFloatWin := strToFloat(nombre);
end;

function FloatToStrPoint(valeur: double): strUnite;
var
   nombre: strUnite;
   posSeparator: integer;
begin
   nombre := FloatToStrF(valeur, ffGeneral, precision, 2);
   if decimalSeparator <> '.' then begin
      posSeparator := pos(decimalSeparator, nombre);
      if posSeparator > 0 then
         Nombre[posSeparator] := '.';
   end;
   result := nombre;
end;

function ConvertitChaine(nombre: strUnite; var valeur: double): boolean;
begin
   if (upcase(nombre[1]) = 'E') then
      nombre := '1' + nombre
   else if nombre[1] = '.' then
      nombre := '0' + nombre;
   convertitChaine := True;
   try
      valeur := StrToFloat(nombre);
   except
      convertitChaine := False;
   end;
end;

function FormatTable(Nombre: double): strUnite;
var
   Nexp: integer;
begin
   if Nombre = sansSignif then begin
      Result := '';
      exit;
   end;
   if abs(Nombre) < 1e-10 then
      Nexp := 2
   else if abs(Nombre) < 1e+10 then
      Nexp := 1
   else
      Nexp := 2;
   Result := FloatToStrF(nombre, ffGeneral, precision, Nexp);
end;

procedure afficheErreur(texte: shortString; code: integer);
var
   SauveCursor: Tcursor;
begin
   SauveCursor := Screen.cursor;
   Screen.cursor := crDefault;
   MessageDlg(texte, mtError, [mbOK], code);
   Screen.cursor := SauveCursor;
end;

function PuissToStr(puiss: integer): strUnite;
var
   dest: strUnite;
   i: integer;
begin
   if puiss = 0 then
      dest := ''
   else
      dest := IntToStr(puiss);
   for i := 1 to length(dest) do
      case dest[i] of
         '0'..'9': dest[i] := chiffreExp[dest[i]];
         '+': dest[i] := PlusExp;
         '-': dest[i] := MoinsExp;
      end;
   PuissToStr := dest;
end;

function Pad(S: shortString; Len: integer): shortstring;
   {-Return a string right-padded to length len with ' '}
var
   o: shortstring;
   Slen: integer;
begin
   Slen := length(S);
   if SLen > Len then
      S := copy(S, 1, Len);
   setLength(o, len);
   Move(S[1], o[1], SLen);
   FillChar(o[Succ(SLen)], Len - SLen, ' ');
   Pad := o;
end;

function CaracToChiffre(carac: char): integer;
begin
   if carac in chiffre then
      Result := Ord(carac) - Ord('0')
   else
      case carac of
         #186: Result := 0;
         #185: Result := 1;
         #178: Result := 2;
         #179: Result := 3;
         #165: Result := 4;
         #169: Result := 5;
         #172: Result := 6;
         #173: Result := 7;
         #174: Result := 8;
         #188: Result := 9;
         else
            Result := 0;
      end;
end;

function IsPortAvailable(ComNum: cardinal): boolean;

   function MakeComName(const Dest: PChar; const ComNum: cardinal): PChar;
      {-Return a string like 'COMXX'}
   begin
      StrFmt(Dest, '\\.\COM%d', [ComNum]);
      MakeComName := Dest;
   end;

var
   ComName: array[0..12] of char;
   Res: integer;
   DeviceLayer: TApdBaseDispatcher;
begin
   DeviceLayer := TApdWin32Dispatcher.Create(nil);
   try
      if (ComNum = 0) then
         Result := False
      else begin
         Res := DeviceLayer.OpenCom(MakeComName(ComName, ComNum), 64, 64);
         if (Res < 0) and (GetLastError <> DWORD(Abs(ecAccessDenied))) then
            Result := False
         else begin
            Result := True;
            DeviceLayer.CloseCom;
         end;
      end;
   finally
      DeviceLayer.Free;
   end;
end;

function AddBackSlash(aPath: string): string;
begin
   Result := EmptyStr;
   if (aPath <> '') and (aPath[Length(aPath)] <> '\') then
      Result := aPath + '\';
end;

// permet de retrouver un repertoire d'aprs sa valeur CSIDL
function GetFolder(CSIDLValue: integer): string;
var
   IdList: PITEMIDLIST;
   Folder: array[0..MAX_PATH] of char;
begin
   if not Failed(SHGetSpecialFolderLocation(0, CSIDLValue, IdList)) then
   begin
      SHGetPathFromIDList(IdList, Folder);
      Result := AddBackSlash(Folder);
   end;
end;

initialization
   winDir := GetFolder(CSIDL_Windows);
   mesDocsDir := GetFolder(CSIDL_Personal);
   if not DirectoryExists(MesDocsDir + 'Regressi') then
      createDir(MesDocsDir + 'Regressi');
   mesDocsDir := MesDocsDir + 'Regressi\';
end.

