{sample program for JanH WinTab component by Jan Hlavenka, jan@hlavenka.cz}
unit main;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, StdCtrls, Graphics, ExtCtrls,
  ComCtrls, JH_WinTab, JH_WinTab_Const, Gauges, Buttons;

type
  TfmMain = class(TForm)
    lbTop1: TLabel;
    lbTop1a: TLabel;
    lbTop2: TLabel;
    lbTop2a: TLabel;
    statBar: TStatusBar;
    Bevel1: TBevel;
    lbTop3: TLabel;
    lbTop3a: TLabel;
    JanHWinTab: TJanHWinTab;
    lbTop4: TLabel;
    lbTop4a: TLabel;
    gauge: TGauge;
    paintBox: TPaintBox;
    treeView: TTreeView;
    btRefresh: TButton;
    spbtStart: TSpeedButton;
    lbStart: TLabel;
    procedure JanHWinTabStart(Sender: TObject);
    procedure JanHWinTabStop(Sender: TObject);
    procedure JanHWinTabPacket(Sender: TObject; PacketSerial, Handle: HWND; Packet: TTabletPacket);
    procedure JanHWinTabCSRChange(Sender: TObject; PacketSerial, Handle: HWND; Packet: TTabletPacket);
    procedure JanHWinTabProximity(Sender: TObject; ContextHandle: HWND; Enter: Boolean);
    procedure btRefreshClick(Sender: TObject);
    procedure spbtStartClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    NodeItem: TTreeNode;
  public
    procedure Display(name, text: string);
    procedure WriteTopInfo;
    procedure WriteLeftInfo;
  end;

var
  fmMain: TfmMain;

implementation
{$R *.dfm}

//------- Events ---------------------------------------------------------------
procedure TfmMain.JanHWinTabStart(Sender: TObject);
begin
    btRefreshClick(Self);
    lbStart.Caption:='Stop';
    spbtStart.Hint:='STOP';
    gauge.MinValue:=JanHWinTab.Pressure.axMin;
    gauge.MaxValue:=JanHWinTab.Pressure.axMax;
end;

procedure TfmMain.JanHWinTabStop(Sender: TObject);
begin
    btRefreshClick(Self);
    lbStart.Caption:='Start';
    spbtStart.Hint:='START';    
end;

procedure TfmMain.JanHWinTabPacket(Sender: TObject; PacketSerial, Handle: HWND; Packet: TTabletPacket);
var Rect: TRect;
    TempTop, TempLeft, PenSize: integer;
    PenColor: TColor;
begin
   statBar.Panels[1].Text:=inttostr(Packet.pkPosition.X)+':'+inttostr(Packet.pkPosition.Y);
   statBar.Panels[2].Text:='Pressure: '+inttostr(Packet.pkPressure);
   statBar.Panels[3].Text:='Button: '+Inttostr(Packet.pkButtons);
   statBar.Panels[4].Text:='Context: '+inttostr(Packet.pkContext);
   statBar.Panels[5].Text:='Status: '+inttostr(Packet.pkStatus);
   gauge.Progress:=Packet.pkPressure;

   case packet.pkButtons of
   1: PenColor:=$00FBAD60;
   2: PenColor:=clWhite;
   3: PenColor:=$00AF67E0;
   4: PenColor:=$0091DFA5;
   5: PenColor:=$0080FFFF;
   6..9: PenColor:=$0060609B;
    else PenColor:=clBlack;
   end;

   if packet.pkStatus>10 then PenColor:=$00C8EECD;  //pen eraser

   paintBox.Canvas.Pen.Color:=PenColor;
   paintBox.Canvas.Brush.Color:=PenColor;
   if packet.pkButtons>0 then begin
      TempLeft:=Mouse.CursorPos.X-fmMain.Left-paintBox.Left-5;
      TempTop:=Mouse.CursorPos.Y-fmMain.Top-paintBox.Top-25;
      if Packet.pkPressure>0
         then PenSize:=Abs(Packet.pkPressure div 25)
         else PenSize:=5;
      Rect.Left:=TempLeft-PenSize;
      Rect.Right:=TempLeft+PenSize;
      Rect.Top:=TempTop-PenSize;
      Rect.Bottom:=TempTop+PenSize;
      paintBox.Canvas.Ellipse(Rect);
   end;
end;

procedure TfmMain.JanHWinTabCSRChange(Sender: TObject; PacketSerial, Handle: HWND; Packet: TTabletPacket);
begin
   btRefreshClick(Self);
end;

procedure TfmMain.JanHWinTabProximity(Sender: TObject; ContextHandle: HWND;
  Enter: Boolean);
begin
   if Enter
       then statBar.Panels[0].Text:='Pen in tablet area'
       else statBar.Panels[0].Text:='Pen is out'
end;

//-------- Display functions ---------------------------------------------------
procedure TfmMain.Display(name, text: string);
begin
    if text='root'
       then NodeItem:=treeView.Items.Add(treeView.TopItem,name)
       else treeView.Items.AddChild(NodeItem,name+': '+text)
end;

procedure TfmMain.WriteTopInfo;
begin with JanHWinTab do if DeviceName<>''
   then begin
      lbTop1a.Caption:=DeviceName;
      lbTop2a.Caption:=WinTabId;
      lbTop3a.Caption:=CursorName;
      lbTop4a.Caption:=ExtensionName;
   end
   else begin
      lbTop1a.Caption:='';
      lbTop2a.Caption:='';
      lbTop3a.Caption:='';
      lbTop4a.Caption:='';
   end;
end;

procedure TfmMain.WriteLeftInfo;
var Categ, i: integer;
begin
    Display('Interface','root');
    Display('WinTab ID',JanHWinTab.WinTabId);
    Display('Specification version',JanHWinTab.SpecVersion);
    Display('Implementation version',JanHWinTab.ImplVersion);
    Display('Devices',IntToStr(JanHWinTab.NDevices));
    Display('Cursors',IntToStr(JanHWinTab.NCursors));
    Display('Contexts',IntToStr(JanHWinTab.NContexts));
    Display('CTX Options',JanHWinTab.GetInfoAsString(WTI_INTERFACE,IFC_CTXOPTIONS,0));
    Display('CTX Save Size',JanHWinTab.GetInfoAsString(WTI_INTERFACE,IFC_CTXSAVESIZE,0));
    Display('Extensions',JanHWinTab.GetInfoAsString(WTI_INTERFACE,IFC_NEXTENSIONS,0));
    Display('Managers',JanHWinTab.GetInfoAsString(WTI_INTERFACE,IFC_NMANAGERS,0));

    Display('Status information','root');
    Display('Contexts opened',IntToStr(JanHWinTab.Contexts));
    Display('System contexts opened',JanHWinTab.GetInfoAsString(WTI_STATUS,STA_SYSCTXS,0));
    Display('Max. packet rate',IntToStr(JanHWinTab.PktRate)+' Hz');
    Display('Packet Data Mask',JanHWinTab.GetInfoAsString(WTI_STATUS,STA_PKTDATA,0));
    Display('Managers',JanHWinTab.GetInfoAsString(WTI_STATUS,STA_MANAGERS,0));
    Display('System',JanHWinTab.GetInfoAsString(WTI_STATUS,STA_SYSTEM,0));
    Display('Button Use',JanHWinTab.GetInfoAsString(WTI_STATUS,STA_BUTTONUSE,3));
    Display('System Button Use',JanHWinTab.GetInfoAsString(WTI_STATUS,STA_SYSBTNUSE,3));

    for i:=1 to 3 do begin case i of
      1: begin
          Categ:=WTI_DEFSYSCTX;
          Display('Default system context','root');
         end;
      2: begin
          Categ:=WTI_DDCTXS;
          Display('Default contexts','root');
         end;
      3: begin
          Categ:=WTI_DSCTXS;
          Display('Default system contexts','root');
         end;
      else begin
          Categ:=WTI_DEFCONTEXT;
          Display('Default context','root');
      end;
     end; // case
     Display('Context name',JanHWinTab.GetInfoAsString(Categ,CTX_NAME,1));
     Display('Context options',JanHWinTab.GetInfoAsString(Categ,CTX_OPTIONS,0));
     Display('Context status',JanHWinTab.GetInfoAsString(Categ,CTX_STATUS,0));
     Display('Context locks',JanHWinTab.GetInfoAsString(Categ,CTX_LOCKS,0));
     Display('Context message base',JanHWinTab.GetInfoAsString(Categ,CTX_MSGBASE,0));
     Display('Context device',JanHWinTab.GetInfoAsString(Categ,CTX_DEVICE,0));
     Display('Context packet rate',JanHWinTab.GetInfoAsString(Categ,CTX_PKTRATE,0));
     Display('Context packet data mask',JanHWinTab.GetInfoAsString(Categ,CTX_PKTDATA,0));
     Display('Context packet mode',JanHWinTab.GetInfoAsString(Categ,CTX_PKTMODE,0));
     Display('Context move mask',JanHWinTab.GetInfoAsString(Categ,CTX_MOVEMASK,0));
     Display('Context button down mask',JanHWinTab.GetInfoAsString(Categ,CTX_BTNDNMASK,3));
     Display('Context button up mask',JanHWinTab.GetInfoAsString(Categ,CTX_BTNUPMASK,3));
     Display('Context origin input area X',JanHWinTab.GetInfoAsString(Categ,CTX_INORGX,0));
     Display('Context origin input area Y',JanHWinTab.GetInfoAsString(Categ,CTX_INORGY,0));
     Display('Context origin input area Z',JanHWinTab.GetInfoAsString(Categ,CTX_INORGZ,0));
     Display('Context extent input area X',JanHWinTab.GetInfoAsString(Categ,CTX_INEXTX,0));
     Display('Context extent input area Y',JanHWinTab.GetInfoAsString(Categ,CTX_INEXTY,0));
     Display('Context extent input area Z',JanHWinTab.GetInfoAsString(Categ,CTX_INEXTZ,0));
     Display('Context origin output area X',JanHWinTab.GetInfoAsString(Categ,CTX_OUTORGX,0));
     Display('Context origin output area Y',JanHWinTab.GetInfoAsString(Categ,CTX_OUTORGY,0));
     Display('Context origin output area Z',JanHWinTab.GetInfoAsString(Categ,CTX_OUTORGZ,0));
     Display('Context extent output area X',JanHWinTab.GetInfoAsString(Categ,CTX_OUTEXTX,0));
     Display('Context extent output area Y',JanHWinTab.GetInfoAsString(Categ,CTX_OUTEXTY,0));
     Display('Context extent output area Z',JanHWinTab.GetInfoAsString(Categ,CTX_OUTEXTZ,0));
     Display('Context sensitivity X',JanHWinTab.GetInfoAsString(Categ,CTX_SENSX,0));
     Display('Context sensitivity Y',JanHWinTab.GetInfoAsString(Categ,CTX_SENSY,0));
     Display('Context sensitivity Z',JanHWinTab.GetInfoAsString(Categ,CTX_SENSZ,0));
     Display('Default system cursor mode',JanHWinTab.GetInfoAsString(Categ,CTX_SYSMODE,0));
     Display('System origin X',JanHWinTab.GetInfoAsString(Categ,CTX_SYSORGX,0));
     Display('System origin Y',JanHWinTab.GetInfoAsString(Categ,CTX_SYSORGY,0));
     Display('System size X',JanHWinTab.GetInfoAsString(Categ,CTX_SYSEXTX,0));
     Display('System size Y',JanHWinTab.GetInfoAsString(Categ,CTX_SYSEXTY,0));
     Display('System sensitivity X',JanHWinTab.GetInfoAsString(Categ,CTX_SYSSENSX,0));
     Display('System sensitivity Y',JanHWinTab.GetInfoAsString(Categ,CTX_SYSSENSY,0));
    end;

    Display('Device information','root');
    Display('Device name',JanHWinTab.DeviceName);
    Display('Hardware flags',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_HARDWARE,0));
    Display('Cursor types',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_NCSRTYPES,0));
    Display('First cursor type',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_FIRSTCSR,0));
    Display('Max. packet rate',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_PKTRATE,0));
    Display('Packet data',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_PKTDATA,0));
    Display('Packet mode',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_PKTMODE,0));
    Display('Cursor mask',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_CSRDATA,0));
    Display('Tablet margin X',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_XMARGIN,0));
    Display('Tablet margin Y',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_YMARGIN,0));
    Display('Tablet margin Z',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_ZMARGIN,0));
    Display('Plug & Play ID',JanHWinTab.GetInfoAsString(WTI_DEVICES,DVC_PNPID,1));

    Display('Cursors information','root');
    Display('Cursor name',JanHWinTab.CursorName);
    Display('Now connected',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_ACTIVE,0));
    Display('Packet data',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_PKTDATA,0));
    Display('Buttons count',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_BUTTONS,0));
    Display('Button data bits',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_BUTTONBITS,0));
    Display('Button names',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_BTNNAMES,1));
    Display('Btn physical No. normal press.',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_NPBUTTON,0));
    Display('Btn physical No. tangent press.',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_TPBUTTON,0));
    Display('Manufacturer specific ID',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_PHYSID,3));
    Display('Cursor mode',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_MODE,0));
    Display('Minimum set of data',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_MINPKTDATA,0));
    Display('Minimum buttons',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_MINBUTTONS,0));
    Display('Capabilities flag',JanHWinTab.GetInfoAsString(WTI_CURSORS,CSR_CAPABILITIES,0));

    Display('Extensions','root');
    Display('Extension name',JanHWinTab.ExtensionName);
    Display('Unique ext. identifier',JanHWinTab.GetInfoAsString(WTI_EXTENSIONS,EXT_TAG,0));
    Display('Extension mask',JanHWinTab.GetInfoAsString(WTI_EXTENSIONS,EXT_MASK,0));                   
end;

procedure TfmMain.btRefreshClick(Sender: TObject);
begin
    treeView.Items.Clear;
    WriteTopInfo;
    WriteLeftInfo;
end;

procedure TfmMain.spbtStartClick(Sender: TObject);
begin
    if JanHWinTab.Active
       then JanHWinTab.Active:=false
       else JanHWinTab.Active:=true;
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
    btRefreshClick(Self)
end;

procedure TfmMain.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    if Key=VK_RETURN then spbtStartClick(Self)
end;

end.

